(***********************************************************************)
(*                                                                     *)
(*            The Symbolic Communicating Machines Library              *)
(*                                                                     *)
(*                Bertrand Jeannet and Tristan Le Gall                 *)
(*                                                                     *)
(*  Copyright 2008 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the GNU Library General Public License, with    *)
(*  the special exception on linking described in file LICENSE.        *)
(*                                                                     *)
(***********************************************************************)

(* SCM Library without Apron: main file *)

let version = "0.2beta - no apron"



open Format
open StandardSem

(* differents analyseurs *)

module CustomBidon =
  struct
    type tlattice = Scm_manager.bidon

  end

type topt_analysis =
    {
      mutable tlattice : string ;
      mutable forward : int ;
      mutable backward : int ;
      mutable threshold : bool ;
      mutable obs : string
    }

module AnalyzerBidon = Scm_analyzer.Make(CustomBidon)


exception Bad_option of string

exception Help

exception Version

let _ =
  let opt_analysis =
    {
      tlattice = "" ;
      forward = 0 ;
      backward = 0 ;
      threshold = false ;
      obs = "" ;
    }
  in
  let i = ref 1 in
    try
      begin
	while !i < Array.length Sys.argv do
	  (match Sys.argv.(!i) with
	      "-v" -> raise Version
	    | "-help" -> raise Help
	    | "-threshold" -> opt_analysis.threshold <- true
	    | "-forward" ->
		begin
		  if !i < (Array.length Sys.argv) -1
		  then (incr i ; opt_analysis.forward <- int_of_string Sys.argv.(!i))
		  else failwith "-forward : integer expected"
		end
	    | "-backward" ->
		begin
		  if !i < (Array.length Sys.argv) -1
		  then (incr i ; opt_analysis.backward <- int_of_string Sys.argv.(!i))
		  else failwith "-backward : integer expected"
		end
	    | "-apron" ->
		begin
		  if !i < (Array.length Sys.argv) -1
		  then (incr i ; 
			match Sys.argv.(!i) with
			    "polka" -> opt_analysis.tlattice <- "polka"
			  | "oct" -> opt_analysis.tlattice <- "oct"
			  | "box" -> opt_analysis.tlattice <- "box"
			  | _ -> failwith "-apron : 'polka', 'box' or 'oct' expected"
		       )
		  else failwith "-apron : 'polka', 'box' or 'oct' expected"
		end
	    | s -> raise (Bad_option s))
	  ;
	  incr i
	done ;
	Format.printf "Lecture du SCM :@." ;

	let scm_lu = Frontend.parse_lexbuf Format.std_formatter (Lexing.from_channel stdin) in

	let scm_glob = Scm_prod.from_syntree scm_lu in 
	  Scm_syn.check_global_scm scm_glob ;
	  let scm_glob = 
	    if opt_analysis.obs = ""
	    then scm_glob 
	    else 
	      begin
		let ch_fichier = open_in  opt_analysis.obs in (* fichier obs *)
		let obs_lu = Frontend.parse_lexbuf Format.std_formatter (Lexing.from_channel ch_fichier) in
		  close_in ch_fichier ;

		  let obs_glob = Scm_prod.from_syntree obs_lu in 
		    Scm_prod.synchronous_product
		      obs_glob scm_glob
	      end
	  in

	  match opt_analysis.tlattice with
	      "" -> 
		let manager = 
		  Scm_manager.finite_manager 
		    ~k_forward:opt_analysis.forward 
		    ~k_backward:opt_analysis.backward 
		    () 
		    scm_glob 
		in
		  AnalyzerBidon.analyze scm_glob manager

	    | _ -> failwith "APRON is NOT supported"
      end
    with
	Bad_option s -> Format.fprintf Format.err_formatter "Unknown option : %s" s
      |	Help -> 
	  Format.pp_print_string
	    Format.std_formatter 
	    "SCM analyzer : option -v, -help, -forward k, -backward k, -threshold"
      | Version ->
	  Format.printf
	    "SCM analyzer - version %s @." 
	    version
