(***********************************************************************)
(*                                                                     *)
(*            The Symbolic Communicating Machines Library              *)
(*                                                                     *)
(*                Bertrand Jeannet and Tristan Le Gall                 *)
(*                                                                     *)
(*  Copyright 2008 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the GNU Library General Public License, with    *)
(*  the special exception on linking described in file LICENSE.        *)
(*                                                                     *)
(***********************************************************************)

(** SCM Library without Apron: abstract manager *)

type environment = unit

type 'abstract manager = 
    {
      env_var : environment ;
      env_param : environment ;
      env_global : environment ;
      get_env : 'abstract -> environment ;     

      (* lattice part *)
      bottom:environment  -> 'abstract ;
      top: environment -> 'abstract ;
      is_bottom : 'abstract -> bool ;
      is_top:  'abstract -> bool ;
      is_leq: 'abstract -> 'abstract -> bool ;
      join: 'abstract -> 'abstract -> 'abstract ;
      join_list: 'abstract list  -> 'abstract ;
      meet: 'abstract -> 'abstract -> 'abstract ;
      widening: 'abstract -> 'abstract -> 'abstract ;
      print_abs: Format.formatter -> 'abstract -> unit ;
      print_env :  Format.formatter -> 'abstract -> unit ;
 
  (* change in the environment 
     global warning : save the earth
  *)
      param_to_glob: 'abstract -> 'abstract ;
      var_to_glob: 'abstract -> 'abstract ;
      glob_to_param: 'abstract -> 'abstract ;
      glob_to_var : 'abstract -> 'abstract ;

(* for the non-standard semantics 
   - glob_to_globprime and globprime_to_primebis change the environment
   - p_to_pprime (resp primebis_to_glob) rename the parameters (resp the variables)
*)
      p_to_pprime : 'abstract -> 'abstract ;
      glob_to_globprime :  'abstract -> 'abstract ;
      globprime_to_primebis :  'abstract -> 'abstract ;
      primebis_to_glob :  'abstract -> 'abstract ;

(* depending on the scm analyzed *)
      from_guard : ?nonstd:bool -> Scm_syn.bexpr -> 'abstract ;
      assign_list : ?nonstd:bool -> 'abstract -> (Scm_syn.var * Scm_syn.iexpr) list -> 'abstract ;
      (* environments : env_global -> env_global or, if nonstd, env_globprime -> env_globprime *)
      rev_assign_list : ?nonstd:bool -> 'abstract -> (Scm_syn.var * Scm_syn.iexpr) list -> 'abstract ;
      (* environments : env_var -> env_global or, if nonstd, env_global -> env_globprime *)
      mutable nbqueues : int    ;
      mutable list_param : string list ;
      mutable list_variables: string list ;
      mutable k_forward : int ; (* parameter of the standard widening *)
      mutable k_backward : int ; (* parameter of the standard widening *)
    }

type bidon = Bot | Top

module type Bidon =
sig
  type t = bidon
      
  val bottom : t

  val top : t

  val is_bottom : t -> bool

  val is_top : t -> bool 

  val is_leq : t -> t -> bool
    
  val join  : t -> t -> t

  val join_list : t list -> t 

  val meet : t -> t -> t

  val widening   : t -> t -> t

  val  print : Format.formatter -> t -> unit
   
end

val finite_manager : 
  ?k_backward:int ->  
  ?k_forward:int -> 
  unit -> 
  Scm_syn.global_scm -> 
  bidon manager
  (* creates a manager adapted to the given SCM. Only for the analysis of CFSM *)
