(***********************************************************************)
(*                                                                     *)
(*            The Symbolic Communicating Machines Library              *)
(*                                                                     *)
(*                Bertrand Jeannet and Tristan Le Gall                 *)
(*                                                                     *)
(*  Copyright 2008 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the GNU Library General Public License, with    *)
(*  the special exception on linking described in file LICENSE.        *)
(*                                                                     *)
(***********************************************************************)

(** SCM controller: main file *)

let version = "0.1 beta"


open Apron
open Format
open StandardSem

(* differents analyseurs *)

type apronPolka = Polka.loose Polka.t
type apronBox = Box.t
type apronOct = Oct.t

module CustomBidon =
  struct
    type tabstract = Scm_manager.bidon

  end

module CustomPolka =
  struct
    type tabstract = apronPolka Abstract1.t 

  end

module CustomBox =
  struct
    type tabstract = apronBox Abstract1.t 

  end

module CustomOct =
  struct
    type tabstract = apronOct Abstract1.t 

  end

type topt_analysis =
    {
      mutable tabstract : string ;
      mutable forward : int ;
      mutable backward : int ;
      mutable threshold : bool ;
      mutable obs : string ;
      mutable widening_start : int ;
      mutable widening_freq : int ;
      mutable print_step : bool ;
      mutable print_state : bool ;
      mutable print_postpre : bool ;
      mutable forward_analysis: bool;
    }

module AnalyzerBidon = Scm_analyzer.Make(CustomBidon)

module AnalyzerPolka = Scm_analyzer.Make(CustomPolka)

module AnalyzerBox = Scm_analyzer.Make(CustomBox)

module AnalyzerOct = Scm_analyzer.Make(CustomOct)

exception Bad_option of string

exception Help

exception Version


let only_uc (gscm: Scm_syn.global_scm) : Scm_syn.global_scm =
  let f_filter_uc transition = 
    match transition. Scm_syn.action with
	Some (_,Scm_syn.RECEIVE,_) -> true
      | _ -> false

  in
  let f_map_states_uc state =
    {
      Scm_syn.id_state = state.Scm_syn.id_state;
      Scm_syn.ltrans = List.filter f_filter_uc state.Scm_syn.ltrans 
    }
  in
    {
      Scm_syn.gb_id_scm = gscm.Scm_syn.gb_id_scm ;
      Scm_syn.gb_nb_channels = gscm.Scm_syn.gb_nb_channels ;
      Scm_syn.gb_lossy_list =  gscm.Scm_syn.gb_lossy_list ;
      Scm_syn.gb_vars =  gscm.Scm_syn.gb_vars ;
      Scm_syn.gb_params = gscm.Scm_syn.gb_params ;
      Scm_syn.gb_init_states = gscm.Scm_syn.gb_init_states ;
      Scm_syn.gb_lstates = List.map f_map_states_uc gscm.Scm_syn.gb_lstates ;
      Scm_syn.gb_threshold = gscm.Scm_syn.gb_threshold ;
      Scm_syn.gb_bad_states = gscm.Scm_syn.gb_bad_states
    }

(* useless 
let is_substring string1 string2 =
  let l1 = String.length string1
  and l2 = string.length string2
  in

    if l1>l2
    then 
      false
    else
      begin
      let res = ref false
      and i = ref 0
      in

	while (not !res) && (!i<= l2-l1) do
	  if String.sub string2 !i l1 = string1
	  then
	    res := true
	  else
	    incr i
	done;
	!res  
      end
*)

let test_forbid_delta 
    (mappe_reach:(string,'a)Mappe.t)
    (mappe_coreach:(string,'a)Mappe.t)
    (post: Scm_syn.transition-> 'a ->'a)
    (inter: 'a -> 'a -> 'a)
    (is_bottom: 'a -> bool)
    (state1:string) 
    (trans:Scm_syn.transition) 
    : bool
    =
  let res = ref false in 

  let f_iter2 post_abs_reach s abs_coreach =
    if 
      (not !res)
      &&
	(trans.Scm_syn.new_state = s )
	(* verifie qu'on arrive bien dans l'état correspondant*)
      &&
	(not (is_bottom (inter (inter (try Mappe.find s mappe_reach with Not_found ->failwith "erreur là") abs_coreach) post_abs_reach)))
   	(* vérifie qu'on a une intersection non vide avec coreach_uc inter reach *)
    then
      res := true

  in

  let f_iter1 s abs_reach =
    if 
      (not !res)
      &&
	(state1 = s )
	(* verifie qu'on arrive bien dans l'état de départ de la transition*)
    then
      Mappe.iter 
	(f_iter2 (post trans abs_reach))
	mappe_coreach

  in
    Mappe.iter f_iter1 mappe_reach ;

    !res

let print_forbidden_action_state 
    (mappe_reach:(string,'a)Mappe.t)
    (mappe_coreach:(string,'a)Mappe.t)
    (post: Scm_syn.transition-> 'a ->'a)
    (inter: 'a -> 'a -> 'a)
    (is_bottom: 'a -> bool)
    fmt 
    (state:Scm_syn.state) 
    :unit
    =
  Format.fprintf 
    fmt 
    "__________________@. state %s : @." 
    state.Scm_syn.id_state ;
  List.iter
    (fun trans -> 
       if	   
	     test_forbid_delta    
	       mappe_reach
	       mappe_coreach
	       post
	       inter
	       is_bottom
	       state.Scm_syn.id_state
	       trans
	   
       then
	 Format.fprintf fmt "%a @." Scm_print.print_transition trans
    )
    state.Scm_syn.ltrans ;
  Format.fprintf fmt "__________________@."













(*  MAIN PART *)


let _ =
  let opt_analysis =
    {
      tabstract = "" ;
      forward = 0 ;
      backward = 0 ;
      threshold = false ;
      obs = "" ;
      widening_start = 1 ;
      widening_freq = 1 ;
      print_step = false ;
      print_state = false ;
      print_postpre = false ;
      forward_analysis = true;
    }
  in
  let i = ref 1 in
    try
      begin
	while !i < Array.length Sys.argv do
	  (match Sys.argv.(!i) with
	       "-v" -> raise Version
	     | "-help" -> raise Help
	     | "-threshold" -> opt_analysis.threshold <- true
	     | "-print_step" -> opt_analysis.print_step <- true
	     | "-print_state" -> opt_analysis.print_state <- true
	     | "-print_postpre" -> opt_analysis.print_postpre <- true
	     | "-backward_analysis" -> opt_analysis.forward_analysis <- false
	     | "-forward" ->
		 begin
		   if !i < (Array.length Sys.argv) -1
		   then (incr i ; try opt_analysis.forward <- int_of_string Sys.argv.(!i) with Failure "int_of_string" -> failwith "-forward : integer expected")
		   else failwith "-forward : integer expected"
		 end
	     | "-backward" ->
		 begin
		   if !i < (Array.length Sys.argv) -1
		   then (incr i ; try opt_analysis.backward <- int_of_string Sys.argv.(!i)with Failure "int_of_string" -> failwith "-backward : integer expected")
		   else failwith "-backward : integer expected"
		 end
	     | "-widening_start" ->
		 begin
		   if !i < (Array.length Sys.argv) -1
		   then (incr i ; try opt_analysis.widening_start <- int_of_string Sys.argv.(!i) with Failure "int_of_string" -> failwith "-widening_start: integer expected")
		   else failwith "-widening_start : integer expected"
		 end
	     | "-widening_freq" ->
		 begin
		   if !i < (Array.length Sys.argv) -1
		   then (incr i ; try opt_analysis.widening_freq <- int_of_string Sys.argv.(!i) with Failure "int_of_string" -> failwith "-widening_freq : integer expected")
		   else failwith "-widening_freq : integer expected"
		 end

	     | "-apron" ->
		 begin
		   if !i < (Array.length Sys.argv) -1
		   then (incr i ; 
			 match Sys.argv.(!i) with
			     "polka" -> opt_analysis.tabstract <- "polka"
			   | "oct" -> opt_analysis.tabstract <- "oct"
			   | "box" -> opt_analysis.tabstract <- "box"
			   | _ -> failwith "-apron : 'polka', 'box' or 'oct' expected"
			)
		   else failwith "-apron : 'polka', 'box' or 'oct' expected"
		 end
	     | s -> raise (Bad_option s))
	  ;
	  incr i
	done ;
	Format.printf "Lecture du SCM @." ;

	let scm_lu = Frontend.parse_lexbuf Format.std_formatter (Lexing.from_channel stdin) in

	let scm_glob = Scm_prod.from_syntree scm_lu in 
	  Scm_syn.check_global_scm scm_glob ;




	  match opt_analysis.tabstract with
	      "" -> 
		begin
		  let manager = 
		    Scm_manager.finite_manager 
		      ~k_forward:opt_analysis.forward 
		      ~k_backward:opt_analysis.backward 
		      () 
		      scm_glob 
		  in
		  let partition = AnalyzerBidon.AbsSem.create_partition manager
		  and lattice = AnalyzerBidon.AbsSem.create_lattice manager

		  in
		  let coreach_uc =
		    AnalyzerBidon.analyze 
		      ~forward:false
		      ~widening_start:opt_analysis.widening_start
		      ~widening_freq:opt_analysis.widening_freq
		      ~print_step:opt_analysis.print_step
		      ~print_state:opt_analysis.print_state
		      ~print_postpre:opt_analysis.print_postpre
		      manager 
		      partition
		      lattice		     
		      (only_uc scm_glob)
		  and reach =
		    AnalyzerBidon.analyze 
		      ~forward:true
		      ~widening_start:opt_analysis.widening_start
		      ~widening_freq:opt_analysis.widening_freq
		      ~print_step:opt_analysis.print_step
		      ~print_state:opt_analysis.print_state
		      ~print_postpre:opt_analysis.print_postpre
		      manager 
		      partition
		      lattice
		      scm_glob
		  in 
		    Format.printf "@. Transitions à interdire: @.";
		    List.iter
		      (fun state ->
			 print_forbidden_action_state 
			   reach
			   coreach_uc
			   (* fonction post *)
			   (fun trans abs -> 
			      begin
				match
				  trans.Scm_syn.action 
				with
				    None ->
				       AnalyzerBidon.AbsSem.epsilon_trans
					 manager
					 trans.Scm_syn.guard
					 trans.Scm_syn.assignement
					 abs
				  | Some (nbq,act,p)->
					 AnalyzerBidon.AbsSem.output_trans 
					   manager
					   trans.Scm_syn.guard
					   (AnalyzerBidon.AbsSem.Alpha.from_string p)
					   nbq
					   trans.Scm_syn.assignement
					   abs
			      end
			   )
			   (AnalyzerBidon.AbsSem.meet manager)
			   (AnalyzerBidon.AbsSem.is_bottom manager)
			   Format.std_formatter
			   state
		      )
		      
		      scm_glob.Scm_syn.gb_lstates ;
		    Format.printf "@. Terminé @."
		      
		end



	    | "polka" ->
		begin
		  let manager = 
		    Scm_manager.apron_manager 		    
		      ~k_forward:opt_analysis.forward 
		      ~k_backward:opt_analysis.backward 
		      ~threshold:opt_analysis.threshold
		      (Polka.manager_alloc_loose ())
		      scm_glob 
		  in
		  let partition = AnalyzerPolka.AbsSem.create_partition manager
		  and lattice = AnalyzerPolka.AbsSem.create_lattice manager

		  in
		  let coreach_uc =
		    AnalyzerPolka.analyze 
		      ~forward:false
		      ~widening_start:opt_analysis.widening_start
		      ~widening_freq:opt_analysis.widening_freq
		      ~print_step:opt_analysis.print_step
		      ~print_state:opt_analysis.print_state
		      ~print_postpre:opt_analysis.print_postpre
		      manager 
		      partition
		      lattice
		      (only_uc scm_glob)
		  and reach =
		    AnalyzerPolka.analyze 
		      ~forward:true
		      ~widening_start:opt_analysis.widening_start
		      ~widening_freq:opt_analysis.widening_freq
		      ~print_step:opt_analysis.print_step
		      ~print_state:opt_analysis.print_state
		      ~print_postpre:opt_analysis.print_postpre
		      manager
 		      partition
		      lattice
		      scm_glob
		  in 
		    Format.printf "@. Transitions à interdire: @.";
		    List.iter
		      (fun state ->
			 print_forbidden_action_state 
			   reach
			   coreach_uc
			   (* fonction post *)
			   (fun trans abs -> 
			      begin
				match
				  trans.Scm_syn.action 
				with
				    None ->
				       AnalyzerPolka.AbsSem.epsilon_trans
					 manager
					 trans.Scm_syn.guard
					 trans.Scm_syn.assignement
					 abs
				  | Some (nbq,act,p) when act= Scm_syn.SEND->
					 AnalyzerPolka.AbsSem.output_trans 
					   manager
					   trans.Scm_syn.guard
					   (AnalyzerPolka.AbsSem.Alpha.from_string p)
					   nbq
					   trans.Scm_syn.assignement
					   abs
				  | Some (nbq,act,p) when act= Scm_syn.RECEIVE->
					 AnalyzerPolka.AbsSem.input_trans 
					   manager
					   trans.Scm_syn.guard
					   (AnalyzerPolka.AbsSem.Alpha.from_string p)
					   nbq
					   trans.Scm_syn.assignement
					   abs
			      end
			   )
			   (AnalyzerPolka.AbsSem.meet manager)
			   (AnalyzerPolka.AbsSem.is_bottom manager)
			   Format.std_formatter
			   state
		      )
		      
		      scm_glob.Scm_syn.gb_lstates ;
		    Format.printf "@. Terminé @."
      
		      
		end

	    | "box" ->
		begin
		  let manager = 
		    Scm_manager.apron_manager 
		      ~k_forward:opt_analysis.forward 
		      ~k_backward:opt_analysis.backward 
		      ~threshold:opt_analysis.threshold
		      (Box.manager_alloc ())
		      scm_glob 
		  in
		  let partition = AnalyzerBox.AbsSem.create_partition manager
		  and lattice = AnalyzerBox.AbsSem.create_lattice manager

		  in
		  let coreach_uc =
		    AnalyzerBox.analyze 
		      ~forward:false
		      ~widening_start:opt_analysis.widening_start
		      ~widening_freq:opt_analysis.widening_freq
		      ~print_step:opt_analysis.print_step
		      ~print_state:opt_analysis.print_state
		      ~print_postpre:opt_analysis.print_postpre
		      manager 
		      partition
		      lattice
		      (only_uc scm_glob)
		  and reach =
		    AnalyzerBox.analyze 
		      ~forward:true
		      ~widening_start:opt_analysis.widening_start
		      ~widening_freq:opt_analysis.widening_freq
		      ~print_step:opt_analysis.print_step
		      ~print_state:opt_analysis.print_state
		      ~print_postpre:opt_analysis.print_postpre
		      manager 
		      partition
		      lattice
		      scm_glob
		  in 
		    Format.printf "@. Transitions à interdire: @.";
		    List.iter
		      (fun state ->
			 print_forbidden_action_state 
			   reach
			   coreach_uc
			   (* fonction post *)
			   (fun trans abs -> 
			      begin
				match
				  trans.Scm_syn.action 
				with
				    None ->
				       AnalyzerBox.AbsSem.epsilon_trans
					 manager
					 trans.Scm_syn.guard
					 trans.Scm_syn.assignement
					 abs
				  | Some (nbq,act,p) when act=Scm_syn.SEND ->
					 AnalyzerBox.AbsSem.output_trans 
					   manager
					   trans.Scm_syn.guard
					   (AnalyzerBox.AbsSem.Alpha.from_string p)
					   nbq
					   trans.Scm_syn.assignement
					   abs
				  | Some (nbq,act,p) when act=Scm_syn.RECEIVE ->
					 AnalyzerBox.AbsSem.input_trans 
					   manager
					   trans.Scm_syn.guard
					   (AnalyzerBox.AbsSem.Alpha.from_string p)
					   nbq
					   trans.Scm_syn.assignement
					   abs
			      end
			   )
			   (AnalyzerBox.AbsSem.meet manager)
			   (AnalyzerBox.AbsSem.is_bottom manager)
			   Format.std_formatter
			   state
		      )
		      
		      scm_glob.Scm_syn.gb_lstates ;
		    Format.printf "@. Terminé @."
		end



	    | "oct" ->
		begin
		  let manager = 
		    Scm_manager.apron_manager 
		      ~k_forward:opt_analysis.forward 
		      ~k_backward:opt_analysis.backward 
		      ~threshold:opt_analysis.threshold
		      (Oct.manager_alloc ())
		      scm_glob 
		  in
		  let partition = AnalyzerOct.AbsSem.create_partition manager
		  and lattice = AnalyzerOct.AbsSem.create_lattice manager

		  in
		  let coreach_uc =
		    AnalyzerOct.analyze 
		      ~forward:false
		      ~widening_start:opt_analysis.widening_start
		      ~widening_freq:opt_analysis.widening_freq
		      ~print_step:opt_analysis.print_step
		      ~print_state:opt_analysis.print_state
		      ~print_postpre:opt_analysis.print_postpre
		      manager 
		      partition
		      lattice
		      (only_uc scm_glob)
		  and reach =
		    AnalyzerOct.analyze 
		      ~forward:true
		      ~widening_start:opt_analysis.widening_start
		      ~widening_freq:opt_analysis.widening_freq
		      ~print_step:opt_analysis.print_step
		      ~print_state:opt_analysis.print_state
		      ~print_postpre:opt_analysis.print_postpre
		      manager 
		      partition
		      lattice
		      scm_glob
		  in 
		    Format.printf "@. Transitions à interdire: @.";
		    List.iter
		      (fun state ->
			 print_forbidden_action_state 
			   reach
			   coreach_uc
			   (* fonction post *)
			   (fun trans abs -> 
			      begin
				match
				  trans.Scm_syn.action 
				with
				    None ->
				       AnalyzerOct.AbsSem.epsilon_trans
					 manager
					 trans.Scm_syn.guard
					 trans.Scm_syn.assignement
					 abs
				  | Some (nbq,act,p) when act = Scm_syn.SEND->
					 AnalyzerOct.AbsSem.output_trans 
					   manager
					   trans.Scm_syn.guard
					   (AnalyzerOct.AbsSem.Alpha.from_string p)
					   nbq
					   trans.Scm_syn.assignement
					   abs
				  | Some (nbq,act,p) when act = Scm_syn.RECEIVE->
					 AnalyzerOct.AbsSem.input_trans 
					   manager
					   trans.Scm_syn.guard
					   (AnalyzerOct.AbsSem.Alpha.from_string p)
					   nbq
					   trans.Scm_syn.assignement
					   abs

			      end
			   )
			   (AnalyzerOct.AbsSem.meet manager)
			   (AnalyzerOct.AbsSem.is_bottom manager)
			   Format.std_formatter
			   state
		      )
		      
		      scm_glob.Scm_syn.gb_lstates ;
		    Format.printf "@. Terminé @."

		end

	    | _ -> failwith "cas impossible"
		
      end
    with
	Bad_option s -> Format.fprintf Format.std_formatter "Unknown option : %s" s
      |	Help -> 
	  Format.pp_print_string
	    Format.std_formatter 
	    "SCM analyzer : option -v, -apron type, -help, -forward k, -backward k, -threshold -backward_analysis @."
      | Version ->
	  Format.printf
	    "SCM analyzer - version %s @." 
	    version
