(***********************************************************************)
(*                                                                     *)
(*            The Symbolic Communicating Machines Library              *)
(*                                                                     *)
(*                Bertrand Jeannet and Tristan Le Gall                 *)
(*                                                                     *)
(*  Copyright 2008 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the GNU Library General Public License, with    *)
(*  the special exception on linking described in file LICENSE.        *)
(*                                                                     *)
(***********************************************************************)

open Scm_syn 
open Format
open StandardSem

  module Alpha =
    struct
      type t = string
      let compare = compare
      let from_string x = x
      let print = Format.pp_print_string
    end 

module AbsSem = StandardSem.Make(Alpha)

let scm_bidon = 
  {
    gb_id_scm = "";
    gb_nb_channels = 3 ;
    gb_lossy_list = [];
    gb_vars= [] ;
    gb_params = [ ("a",Int,None) ; ("b",Int,None)] ;
    gb_init_states = [] ;
    gb_lstates = [] ;
    gb_threshold = [] ;
    gb_bad_states = []
  }

let man = Scm_manager.finite_manager () scm_bidon

let partition = AbsSem.create_partition man

let lattice = AbsSem.create_lattice man

let abs0 = AbsSem.init_abstr man partition lattice []

let abs1 = AbsSem.output_trans man TRUE "a" 1 [] abs0

let abs2 = AbsSem.output_trans man TRUE "b" 1 [] abs0

let abs3 = AbsSem.join man abs1 abs2

let abs4 = AbsSem.pre_output_trans man TRUE "a" 1 [] abs1

let abs5 = AbsSem.pre_output_trans man TRUE "a" 1 [] abs2

let abs6 = AbsSem.pre_output_trans man TRUE "a" 1 [] abs3

let abs7 = AbsSem.pre_input_trans man TRUE "a" 1 [] abs1

let abs8 = AbsSem.pre_input_trans man TRUE "a" 1 [] abs2

let abs9 = AbsSem.pre_input_trans man TRUE "a" 1 [] abs3

let abs10 = AbsSem.output_trans man TRUE "a" 1 [] abs9

let abs11 = AbsSem.output_trans man TRUE "b" 1 [] abs10

let abs12 = AbsSem.meet man abs11 abs7

let abs13 = AbsSem.pre_output_trans man TRUE "a" 1 [] abs10

let abs14 = AbsSem.pre_output_trans man TRUE "a" 1 [] abs11

let abs15 = AbsSem.pre_output_trans man TRUE "a" 1 [] abs12

let _ =
  printf "eps : @. @[ %a @] @." (AbsSem.print man) abs0 ;
  printf "a : @. @[ %a @] @." (AbsSem.print man) abs1 ;
  printf "b : @. @[ %a @] @." (AbsSem.print man) abs2 ;
  printf "a+b : @. @[ %a @] @." (AbsSem.print man) abs3 ;
  printf "eps : @. @[ %a @] @." (AbsSem.print man) abs4 ;
  printf "bot : @. @[ %a @] @." (AbsSem.print man) abs5 ;
  printf "eps : @. @[ %a @] @." (AbsSem.print man) abs6 ;
  printf "aa : @. @[ %a @] @." (AbsSem.print man) abs7 ;
  printf "ab : @. @[ %a @] @." (AbsSem.print man) abs8 ;
  printf "a(a+b) : @. @[ %a @] @." (AbsSem.print man) abs9 ;
  printf "a(a+b)a : @. @[ %a @] @." (AbsSem.print man) abs10 ;
  printf "a(a+b)ab : @. @[ %a @] @." (AbsSem.print man) abs11 ;
  printf "bot : @. @[ %a @] @." (AbsSem.print man) abs12 ;
  printf "a(a+b) : @. @[ %a @] @." (AbsSem.print man) abs13 ;
  printf "bot : @. @[ %a @] @." (AbsSem.print man) abs14 ;
  printf "bot : @. @[ %a @] @." (AbsSem.print man) abs15 ;
 printf "Tests termins ! @."


let scm_bidon1 = 
  {
    gb_id_scm = "";
    gb_nb_channels = 1 ;
    gb_lossy_list = [];
    gb_vars= [] ;
    gb_params = [ ("m",Int,None)] ;
    gb_init_states = [] ;
    gb_lstates = [] ;
    gb_threshold = [] ;
    gb_bad_states = []
  }

let man = Scm_manager.finite_manager () scm_bidon1

let partition = AbsSem.create_partition man

let lattice = AbsSem.create_lattice man

let top = AbsSem.top man partition lattice

let bottom = AbsSem.bottom man partition lattice

let init = AbsSem.init_abstr man partition lattice []

let abs1 = AbsSem.output_trans man TRUE "m" 0 [] init

let abs2 = AbsSem.output_trans man TRUE "m" 0 [] abs1

let abs2c = AbsSem.complementary man abs2

let abs2top = AbsSem.join man abs2 abs2c

let abs2bot = AbsSem.meet man abs2 abs2c

let abs3 = AbsSem.join man abs1 abs2

let abs4 = AbsSem.widening man abs2 abs3

let abs4c = AbsSem.complementary man abs4

let abs4top = AbsSem.join man abs4 abs4c

let abs4bot = AbsSem.meet man abs4 abs4c

let abs5 = AbsSem.join man init abs4
let abs5c = AbsSem.complementary man abs5

let abs6 = AbsSem.meet man (AbsSem.complementary man init)  (AbsSem.complementary man abs4)


let abs7 = AbsSem.pre_output_trans man TRUE "m" 0 [] abs1

let _ =
  assert (AbsSem.is_leq man bottom init);
  assert (AbsSem.is_leq man init top);
  assert (AbsSem.is_leq man bottom top);

  assert (AbsSem.is_leq man bottom abs2);
  assert (AbsSem.is_leq man bottom abs2c);
  assert (AbsSem.is_leq man abs2c top);
  assert (AbsSem.is_leq man abs2 top);

  assert (AbsSem.is_bottom man abs2bot);
  assert (AbsSem.is_top man abs2top);
  assert (AbsSem.is_leq man abs2bot bottom);
  assert (AbsSem.is_leq man top abs2top);

  assert (AbsSem.is_leq man abs1 abs3);
  assert (AbsSem.is_leq man abs2 abs3);
  assert (AbsSem.is_leq man abs3 abs4);

  assert (AbsSem.is_leq man bottom abs4);
  assert (AbsSem.is_leq man bottom abs4c);
  assert (AbsSem.is_leq man abs4c top);
  assert (AbsSem.is_leq man abs4 top);

  assert (AbsSem.is_bottom man abs4bot);
  assert (AbsSem.is_top man abs4top);
  assert (AbsSem.is_leq man abs4bot bottom);
  assert (AbsSem.is_leq man top abs4top);

(*  assert (AbsSem.is_bottom man abs5c); 
  assert (AbsSem.is_leq man abs5c bottom);
  assert (AbsSem.is_leq man top abs5);
  assert (AbsSem.is_leq man abs5c abs6);
  assert (AbsSem.is_leq man abs6 abs5c); *)



  printf "init : @. @[ %a @] @." (AbsSem.print man) init ;
  printf "abs1 : @. @[ %a @] @." (AbsSem.print man) abs1 ;
  printf "abs2 : @. @[ %a @] @." (AbsSem.print man) abs2 ;
  printf "abs3 : @. @[ %a @] @." (AbsSem.print man) abs3 ;
  printf "abs4 : @. @[ %a @] @." (AbsSem.print man) abs4;
  printf "abs5 : @. @[ %a @] @." (AbsSem.print man) abs5;
  printf "abs5c : @. @[ %a @] @." (AbsSem.print man) abs5c;
  printf "abs6 : @. @[ %a @] @." (AbsSem.print man) abs6;
  if AbsSem.is_leq man abs5 top then printf "abs5 <= top @.";
  if AbsSem.is_leq man top abs5 then printf "top <= abs5 @.";
  printf "Tests termins ! @."

let scm_bidon3 = 
  {
    gb_id_scm = "";
    gb_nb_channels = 3 ;
    gb_lossy_list = [];
    gb_vars= [] ;
    gb_params = [ ("i",Int,None) ; ("o",Int,None) ; ("M",Int,None)] ;
    gb_init_states = [] ;
    gb_lstates = [] ;
    gb_threshold = [] ;
    gb_bad_states = []
  }

let man = Scm_manager.finite_manager () scm_bidon3

let partition = AbsSem.create_partition man

let lattice = AbsSem.create_lattice man

let abs0 = AbsSem.init_abstr man partition lattice []

let eps_ioo_eps = AbsSem.output_trans man TRUE "o" 1 [] (AbsSem.output_trans man TRUE "o" 1 [] (AbsSem.output_trans man TRUE "i" 1 [] abs0))

let eps_ooo_eps = AbsSem.output_trans man TRUE "o" 1 [] (AbsSem.output_trans man TRUE "o" 1 [] (AbsSem.output_trans man TRUE "o" 1 [] abs0))

let o_ooooo_eps = AbsSem.output_trans man TRUE "o" 0 [] (AbsSem.output_trans man TRUE "o" 1 [] (AbsSem.output_trans man TRUE "o" 1 [] eps_ooo_eps))

let oo_ooooo_eps = AbsSem.output_trans man TRUE "o" 0 [] o_ooooo_eps

let abs1 = AbsSem.join man (AbsSem.join man eps_ioo_eps  eps_ooo_eps) (AbsSem.join man o_ooooo_eps oo_ooooo_eps)

let abs1c = AbsSem.complementary man abs1

let abs2 =  AbsSem.pre_output_trans man TRUE "o" 0 [] abs1c


let _ =

  printf "init : @. @[ %a @] @." (AbsSem.print man) abs0 ;
  printf "eps_ioo_eps : @. @[ %a @] @." (AbsSem.print man) eps_ioo_eps ;
  printf "eps_ooo_eps : @. @[ %a @] @." (AbsSem.print man) eps_ooo_eps ;
  printf "o_ooooo_eps : @. @[ %a @] @." (AbsSem.print man) o_ooooo_eps ;
  printf "oo_ooooo_eps : @. @[ %a @] @." (AbsSem.print man) oo_ooooo_eps ;
  printf "abs1 : @. @[ %a @] @." (AbsSem.print man) abs1 ;
  printf "abs1c : @. @[ %a @] @." (AbsSem.print man) abs1c;
  printf "abs2 : @. @[ %a @] @." (AbsSem.print man) abs2


let scm_bidon4 = 
  {
    gb_id_scm = "";
    gb_nb_channels = 2 ;
    gb_lossy_list = [];
    gb_vars= [] ;
    gb_params = [ ("m0",Int,None) ; ("m1",Int,None) ; ("a0",Int,None); ("a1",Int,None)] ;
    gb_init_states = [] ;
    gb_lstates = [] ;
    gb_threshold = [] ;
    gb_bad_states = []
  }

let man = Scm_manager.finite_manager () scm_bidon4

let partition = AbsSem.create_partition man

let lattice = AbsSem.create_lattice man

let abs0 = AbsSem.init_abstr man partition lattice []

let abs1 = AbsSem.output_trans man TRUE "m1" 0 [] abs0

let abs2 = AbsSem.output_trans man TRUE "m0" 0 [] abs0

let abs3 = AbsSem.output_trans man TRUE "m0" 0 [] abs2

let abs4 = AbsSem.join man abs2 abs3

let abs5 = AbsSem.widening man abs2 abs4

let abs6 = AbsSem.output_trans man TRUE "m1" 0 [] abs5

let abs7 = AbsSem.join man abs1 abs6

let abs7d = AbsSem.determinise man abs7

let abs7m = AbsSem.minimise man abs7

let abs7dm = AbsSem.minimise man abs7d

let abs8 = AbsSem.complementary man abs7

let abs9 = AbsSem.meet man abs1 abs8

let abs10 = AbsSem.pre_output_trans man TRUE "m1" 0 [] abs8

let abs11 =  AbsSem.meet man abs0 abs10

let _ =
  printf "r : @. @[ %a @] @." (AbsSem.print man) abs0;
  printf "pre(r') : @. @[ %a @] @." (AbsSem.print man) abs10;
  ignore(AbsSem.meet man abs0 abs10);

  printf "r : @. @[ %a @] @." (AbsSem.print man) abs0;
  printf "r' : @. @[ %a @] @." (AbsSem.print man) abs8;
  printf "post(r) : @. @[ %a @] @." (AbsSem.print man) abs1;
  printf "pre(r') : @. @[ %a @] @." (AbsSem.print man) abs10;
  printf "post(r)^r' : @. @[ %a @] @." (AbsSem.print man) abs9;
  printf "pre(r')^r : @. @[ %a @] @." (AbsSem.print man) abs11;
  printf "r : @. @[ %a @] @." (AbsSem.print man) abs0;

