(***********************************************************************)
(*                                                                     *)
(*            The Symbolic Communicating Machines Library              *)
(*                                                                     *)
(*                Bertrand Jeannet and Tristan Le Gall                 *)
(*                                                                     *)
(*  Copyright 2008 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the GNU Library General Public License, with    *)
(*  the special exception on linking described in file LICENSE.        *)
(*                                                                     *)
(***********************************************************************)

(** SCM analyzer: analyzer module *)


(** Alpha is the module of the (finite) alphabet *)
module type Alpha =
sig
  type t
  val compare : t -> t -> int
  val from_string : string -> t
  val print : Format.formatter -> t -> unit
end

(** Custom encapsulates the abstraction of the data *)
module type Custom = sig
  type tabstract

  end

(** Signature of the output of the functor *)
module type S = sig
  module Custom : Custom
  module AbsSem : StandardSem.S
 
  val analyze : 
    ?forward:bool ->
    ?widening_start:int ->
    ?widening_freq:int ->
    ?print_step:bool ->
    ?print_state:bool ->
    ?print_postpre:bool ->
    Custom.tabstract Scm_manager.manager ->
    Custom.tabstract AbsSem.PL.t ->
    Custom.tabstract AbsSem.PL.lattice->
    Scm_syn.global_scm  -> 
    (string,Custom.tabstract AbsSem.t) Mappe.t 
      (** [analyze scm man] performs a reachability analysis (if
	  [forward] is true) or a coreachability analysis (if [forward] is
	  false) and returns a map of (co)reachable states. Other options
	  are standart options of the fixpoint module.

	  The reachability analysis starts from the location labelled
	  as initial in [scm], with an empty queue. The coreachability
	  analysis starts from the bad states of [scm].
      *)


  val check_bad_states :
    ?forward:bool->
    Custom.tabstract Scm_manager.manager ->
    Custom.tabstract AbsSem.PL.t ->
    Custom.tabstract AbsSem.PL.lattice->
    Scm_syn.global_scm  -> 
    (string,Custom.tabstract AbsSem.t) Mappe.t  ->
    bool

(** [check_bad_states scm reach] checks whether bad states are
    reachable (if [forward] is true) or checks whether initial states
    are co-rechable (if [forward] is false).

    [reach] is assumed to be the set of (co)reachable states
    obtained by the function [analyze] 

    NB: if [check_bad_states scm reach] is true, it means that the
    system may be unsafe
*)

  end
  
module Make (Custom:Custom) : (S with module Custom = Custom) 
