(* $Id: LazyAbstraction.mli 1185 2011-10-14 00:37:37Z gsutre $ *)

(*
 * Copyright (c) 2011 CNRS & Université Bordeaux 1.
 *
 * Author(s): Grégoire Sutre <gregoire.sutre@labri.fr>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *)


(**
 * Lazy Abstraction Refinement Tree.
 *
 * This module implements the lazy abstraction approach of McMillan (CAV'06),
 * which is based on the construction of an abstract reachability tree.  In this
 * approach, vertices are created without any environment information, i.e.,
 * they remain labeled with the [top] region until they are refined.  The tree
 * built by the algorithm contains vertex covering information that is used for
 * termination (descendants of covered vertices are not expanded).
 *
 * Several options are provided to tweak the analysis.  In particular, this
 * implementation deviates from the original approach of McMillan (CAV'06) in
 * that vertices being refined with an empty region are removed from the tree.
 * The original approach would keep them in the tree, and an option may be used
 * to restore this behavior.
 *)

(*
 * Note: an intermediate solution would consist in considering these vertices as
 * inactive (e.g., by making them self-covered, and keeping them that way...).
 * However, preliminary testing showed that this solution was systematically
 * worse than the pruning that is done by default.
 *)

module Make : functor (E : TraceChecker.S) -> ModelChecker.S
