(* $Id: TraceChecker.ml 1178 2011-10-13 09:20:22Z gsutre $ *)

(*
 * Copyright (c) 2011 CNRS & Université Bordeaux 1.
 *
 * Author(s): Grégoire Sutre <gregoire.sutre@labri.fr>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *)


open Util


module T (A : Model.SEMANTICS_CLOSURE) =
struct
  module Atom =
  struct
    type 'a t = [
        `State of 'a * A.Region.t
      | `Guard of A.Region.t
      | `Command of A.Command.t
    ]

    let print printer fmt (atom : 'a t) =
      match atom with
          `State (s, r) ->
            Format.fprintf fmt "State@ (@[%a@],@ @[%a@])"
              printer s A.Region.print r
        | `Guard r ->
            Format.fprintf fmt "Guard@ (@[%a@])"
              A.Region.print r
        | `Command cmd ->
            Format.fprintf fmt "Command@ (@[%a@])"
              A.Command.print cmd
  end

  module Trace =
  struct
    type 'a t = 'a Atom.t Sequence.t

    let print printer = Sequence.print "@ ;@ " (Atom.print printer)
  end

  module Result =
  struct
    type 'a invariant = int * (('a * A.Region.t) * A.Region.t) Sequence.t * int

    type 'a t = [
        `Feasible of [ `Strong | `Weak ]
      | `Invariant_forward of 'a invariant
      | `Invariant_backward of 'a invariant
    ]

    let print printer fmt (result : 'a t) =
      let print_invariant_pair fmt ((s, _), r) =
        Format.fprintf fmt "@[<3>%a@]@;@[<3>|->@;%a@]"
          printer s A.Region.print r
      in
        match result with
            `Feasible _ -> Format.fprintf fmt "Feasible"
          | `Invariant_forward (m, s, n) ->
              Format.fprintf fmt "@[<v 3>%s path invariant [%d, %d]:@,%a@]"
                "Forward" m n
                (Sequence.print "@," print_invariant_pair) s
          | `Invariant_backward (m, s, n) ->
              Format.fprintf fmt "@[<v 3>%s path invariant [%d, %d]:@,%a@]"
                "Backward" m n
                (Sequence.print "@," print_invariant_pair) s
  end
end

module type S = functor (A : Model.SEMANTICS_CLOSURE) ->
sig
  include module type of T(A)

  val trace_check : 'a Trace.t -> 'a Result.t
end
