(* $Id: Collection.mli 1133 2011-09-05 11:43:24Z gsutre $ *)

(*
 * Copyright (c) 2009 CNRS & Université Bordeaux 1.
 *
 * Author(s): Grégoire Sutre <gregoire.sutre@labri.fr>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *)


module Set :
sig
  module type S =
  sig
    type elt
    type t
    val is_empty : t -> bool
    val mem : t -> elt -> bool
    val cardinal : t -> int
    val iter : (elt -> unit) -> t -> unit
    val fold : (elt -> 'a -> 'a) -> t -> 'a -> 'a
    val elements : t -> elt list
    val equal : t -> t -> bool
  end

  module type IS =
  sig
    include S
    val create : int -> t
    val clear : t -> unit
    val copy: t -> t
    val add : t -> elt -> unit
    val remove : t -> elt -> unit
  end

  module type PS =
  sig
    include S
    val empty : t
    val add : t -> elt -> t
    val remove : t -> elt -> t
    val filter : (elt -> bool) -> t -> t
  end

  module Imperative :
  sig
    module MakeHashSet (T : Hashtbl.HashedType) : IS with type elt = T.t
    module MakeMutable (Set : PS) : IS with type elt = Set.elt
  end

  module Persistent :
  sig
    module MakeTreeSet (T : Set.OrderedType) : PS with type elt = T.t
  end
end

module Map :
sig
  module type M =
  sig
    type key
    type 'a t
    val is_empty: 'a t -> bool
    val mem:  'a t -> key -> bool
    val find: 'a t -> key -> 'a
    val size : 'a t -> int
    val iter: (key -> 'a -> unit) -> 'a t -> unit
    val fold: (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
  end

  module type IM =
  sig
    include M
    val create: int -> 'a t
    val clear: 'a t -> unit
    val copy: 'a t -> 'a t
    val add: 'a t -> key -> 'a -> unit
    val remove: 'a t -> key -> unit
  end

  module type PM =
  sig
    include M
    val empty : 'a t
    val add: 'a t -> key -> 'a -> 'a t
    val remove: 'a t -> key -> 'a t
  end

  module Imperative :
  sig
    module MakeHashMap (T : Hashtbl.HashedType) : IM with type key = T.t
    module MakeMutable (Map : PM) : IM with type key = Map.key
  end

  module Persistent :
  sig
    module MakeTreeMap (T : Map.OrderedType) : PM with type key = T.t
  end
end
