(* $Id: Sequence.ml 1178 2011-10-13 09:20:22Z gsutre $ *)

(*
 * Copyright (c) 2011 CNRS & Université Bordeaux 1.
 *
 * Author(s): Grégoire Sutre <gregoire.sutre@labri.fr>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *)


type direction = Forward | Backward

type 'a t = 'a list * direction * int

let rev_dir = function Forward -> Backward | Backward -> Forward

let is_empty (_, _, n) = (n = 0)
let length (_, _, n) = n
let rev (l, d, n) = (l, rev_dir d, n)
let forward (l, d, n) =
  match d with Forward -> (l, d, n) | Backward -> (List.rev l, Forward, n)
let backward (l, d, n) =
  match d with Backward -> (l, d, n) | Forward -> (List.rev l, Backward, n)

let print sep printer fmt s =
  let (l, _, _) = forward s
  in
    Format.fprintf fmt "%a"
      (Print.list_printer_from_printer sep printer) l

let map f (l, d, n) = (List.rev_map f l, rev_dir d, n)

let filter p (l, d, n) =
  let rec aux acc_l acc_n =
    function
        [] -> (acc_l, rev_dir d, acc_n)
      | x::l ->
          if p x then
            aux (x::acc_l) (acc_n+1) l
          else
            aux acc_l acc_n l
  in
    aux [] 0 l

let of_array a = (Array.to_list a, Forward, Array.length a)

let to_array (l, d, n) =
  match l with
      [] -> [||]
    | head::tail ->
        let result = Array.make n head in
        let rec fill_fwd i = function
            [] -> ()
          | x::l -> Array.set result i x ; fill_fwd (i+1) l
        and fill_bwd i = function
            [] -> ()
          | x::l -> Array.set result i x ; fill_bwd (i-1) l
        in
          match d with
              Forward -> fill_fwd 1 tail ; result
            | Backward -> fill_bwd (n-2) tail ; result
