(* $Id: Statistics.ml 737 2009-07-17 15:31:41Z sutre $ *)

(*
 * Copyright (c) 2009 CNRS & Université Bordeaux 1.
 *
 * Author(s): Grégoire Sutre <gregoire.sutre@labri.fr>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *)


type statistic = {
  name         : string ;
  start        : float ;
  mutable stop : float ;
}

let stats = ref []

let clear () = stats := []

let register_start name =
  stats := { name  = name ;
             start = Sys.time () ;
             stop  = nan } :: !stats

let register_stop name =
  try
    let s = List.find (fun s -> name = s.name) !stats
    in
      s.stop <- Sys.time ()
  with
      Not_found ->
        raise (Invalid_argument "register_stop")

let print_stat sep fmt s =
  Format.fprintf fmt sep ;
  Format.fprintf fmt "%10.2f s   (%s)" (s.stop -. s.start) s.name

let print fmt () =
    let max_words =                      (* size in words *)
      (Gc.stat ()).Gc.top_heap_words +   (* maximum size reached by the major heap *)
      (Gc.get  ()).Gc.minor_heap_size    (* size of the minor heap *)
    in
      begin
        match (List.rev !stats) with
            [] -> ()
          | head :: tail ->
              Format.fprintf fmt "Execution times: @[<v>" ;
              print_stat "" fmt head ;
              List.iter (print_stat "@," fmt) tail ;
              Format.fprintf fmt "@]"
      end ;
      Format.fprintf fmt "@;Maximum heap size:  %7.2f MiB"
        ((float_of_int (max_words * (Sys.word_size / 8))) /. (2. ** 20.))
