(* $Id: Uid.mli 689 2009-05-08 09:06:01Z sutre $ *)

(*
 * Copyright (c) 2009 CNRS & Université Bordeaux 1.
 *
 * Author(s): Grégoire Sutre <gregoire.sutre@labri.fr>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *)


(**
 * Printable type with unique identifiers.
 *
 * This module provides a functor that embeds a given printable type with an
 * identifier that is guaranteed to be unique for all values of the output
 * printable type.  The identifiers are used to provide constant-time comparison
 * and hashing functions for the output type.  Note that equality of values of
 * the output type implies equality of the underlying values of the input type,
 * but the converse does not hold in general.
 *)

(**
 * Raised when no new identifier is available.
 *)
exception Overflow

(**
 * Functor embedding a given printable type with unique identifiers.
 *)
module Make (T : Print.PRINTABLE_TYPE) :
sig
  include Print.PRINTABLE_TYPE

  val create : T.t -> t
  val value : t -> T.t
  val id : t -> int
  val compare : t -> t -> int
  val equal : t -> t -> bool
  val hash : t -> int

  val lift_una_op : (T.t -> T.t) -> (t -> t)
  val lift_bin_op : (T.t -> T.t -> T.t) -> (t -> t -> t)

  val lift_una_fun : (T.t -> 'a) -> (t -> 'a)
  val lift_bin_fun : (T.t -> T.t -> 'a) -> (t -> t -> 'a)
end
